/*
 * File:  riff_defs.h
 *
 * Definitions and declarations for the Rifficator application
 * 
 */ 
#ifndef  RIFF_DEFS_H
#define  RIFF_DEFS_H
 
#include <avr/io.h>
#include <stdlib.h>
#include <string.h>
#include "Pro_Micro_periph_lib.h"
#include "OLED_display_avr8_lib.h"
#include "MIDI_comms_lib.h"

#ifndef F_CPU
#define F_CPU  16000000UL    // CPU clock frequency
#endif

// Hard-coded constants -- may be changed only by re-compiling the code!
#define MIDI_OUT_CHANNEL  1     // 1..16
#define KEY_VELOCITY      80    // todo: make dynamic - (sample touch ADC value ???)

// Push-buttons 'A', 'B', 'C', 'D' are wired to port pins PB1, PB2, PB3, PB4, resp.
// Macro to configure switch input pins, enable internal pull-ups:
#define SETUP_SWITCH_INPUT_PINS()  { DDRB &= 0b11100001;  PORTB |= 0b00011110; }
    
#define BUTTON_A_PRESSED  ((PINB & (1<<1)) == 0)
#define BUTTON_B_PRESSED  ((PINB & (1<<2)) == 0)
#define BUTTON_C_PRESSED  ((PINB & (1<<3)) == 0)
#define BUTTON_D_PRESSED  ((PINB & (1<<4)) == 0)

#define LEFT_KEY    0     // Touch keypad identifiers
#define CENTRE_KEY  1
#define RIGHT_KEY   2
#define EXTRA_KEY   3

#define LED1_TURN_ON()    SET_BIT(PORTC, 6)
#define LED1_TURN_OFF()   CLEAR_BIT(PORTC, 6)
#define LED2_TURN_ON()    SET_BIT(PORTD, 7)
#define LED2_TURN_OFF()   CLEAR_BIT(PORTD, 7)
#define LED3_TURN_ON()    SET_BIT(PORTE, 6)
#define LED3_TURN_OFF()   CLEAR_BIT(PORTE, 6)

// Valid options for playerMode:
#define RIFF_MODE  0
#define DRUM_MODE  1
#define LOOP_MODE  2

// Valid states for 'PlayerTask' state-machine...
#define KEYS_OFF_OFF_OFF   0x00
#define KEYS_OFF_OFF_ON    0x01
#define KEYS_OFF_ON_OFF    0x02
#define KEYS_OFF_ON_ON     0x03
#define KEYS_ON_OFF_OFF    0x04
#define KEYS_ON_OFF_ON     0x05
#define KEYS_ON_ON_OFF     0x06
#define KEYS_ON_ON_ON      0x07

// Note sequence "direction" may be any one of these:
#define SAME_AGAIN        0x00    // Next note must be same as last (repeat)
#define ANY_DIRN          0x07    // Next note may be higher, lower or same
#define GOING_UP          0x1F    // Next note should be higher (or same)
#define GOING_DOWN        0xFF    // Next note should be lower (or same)


// ------------  Function prototype definitions  ------------------------------
void   PlayerTask();
void   InitiateNote(uint8 key, uint8 dirn);
void   TerminateNote(uint8 key);
uint8  GetCurrentScreenID();
void   GoToNextScreen(uint8 screenID);
void   UserInterfaceTask();
uint8  NoteSequence(uint8 direction);
unsigned  FastRand(unsigned long seed);
uint8  GaussRand(uint8 minval, uint8 maxval, uint8 spread);
uint8  FilteRand(uint8 minval, uint8 maxval, uint8 klag);
void   ButtonScan(unsigned char nb);
bool   ButtonHit(char  button_ID);
void   PotService();
bool   PotMoved();
uint8  PotPosition();
void   KeypadScan();
bool   KeyTouched(uint8 pad);
bool   KeyStruck(uint8 pad);
bool   KeyReleased(uint8 pad);
void   KeypadClear();
void   TouchCalibrate();
void   TouchService();

// Data structure holding persistent parameters
struct  persistent_params
{
	uint8  midi_chan;    // MIDI channel # (1..16)
    uint8  mid_note;     // note in centre of range (MIDI note number)
    uint8  range;        // allowed range of notes (semitones)
    uint8  opt_rand;     // random number gen. option (0: Gaussian, 1: Filter)
    uint8  spread;       // spread of Gaussian distribution (10..250 %)
    uint8  k_damp;       // damping level of filtered random series (1..10)
    uint8  p_accid;      // probability of hitting an accidental (0..100 %)
	uint8  padNote[4];   // midi note numbers assigned to pads 0..3
	//
	long   check;      // constant for EEPROM data integrity check
} ;   

// ------------  Global variables and data  -----------------------------------
//
extern struct persistent_params  param;  // (see above struct def'n)
extern unsigned  task1Freq;     // Call frequency of 1ms task (Hz)

extern uint8  touch_onoff_thres[];  // touch-pad on/off threshold values
extern uint8  touch_reading[];      // touch-pad ADC readings (0..255)
extern uint8  touch_pressure[];     // touch-pad pressure when struck (0..255)

extern uint8  notePlaying[];    // MIDI note numbers (0 => No note playing)
extern uint8  noteKeyedLast;    // MIDI note number of last note keyed
extern uint8  playerMode;       // 0: Riff, 1: Pad Player, 2: Looper
extern uint8  keypadState;      // Keypad state (pads touched)
extern bool   isBlackNote;      // Last note selected was an 'accidental'
extern uint8  randNote;         // Note # last output from random nunber gen.
extern uint8  anyWhiteNote[];   // Table of note numbers in C-major scale

#endif  // RIFF_DEFS_H
